@@ -4,7 +4,7 @@ infinidat jira/confluence release-notes command-line tool
 Usage:
     jirelnotes show {project} [--include-next-release]
     jirelnotes publish {project} [--include-next-release]
-    jirelnotes notify {project} {version}
+    jirelnotes notify {project} {version} [<other-versions-included-in-this-releases>...] [--dry-run]
     jirelnotes fetch
 
 Options:
@@ -15,9 +15,30 @@ Options:
     --project=PROJECT                    project key {project_default}
 """
 
-
 RELEASE_NOTES_TITLE_KEY = 'Release Notes Title'
 RELEASE_NOTES_DESCRIPTION_KEY = 'Release Notes Description'
+NOTIFICATION_MESSAGE = """{{ project.name }} v{{ version }} has been released and now available for download.
+{% if other_versions %}
+This release supercedes the following unreleased versions:
+{%- for version in other_versions %}
+* {{ version }}
+{%- endfor %}
+{%- endif %}
+{% if resolved_issues %}
+This release solves the following related issues:
+{%- for issue in resolved_issues %}
+* {{ issue.key }}: {{ issue_mappings.Summary(issue) }}
+{%- endfor %}
+{%- endif %}
+{% if unresolved_issues %}
+Still, there are more related issues that are still open:
+{%- for issue in unresolved_issues %}
+* {{ issue.key }}: {{ issue_mappings.Summary(issue) }}
+{%- endfor %}
+{%- else %}
+All related issues are now resolved.
+{%- endif %}
+"""
 
 
 def _get_arguments(argv, environ):
@@ -85,7 +106,7 @@ def should_appear_in_release_notes(release):
     return bool(release) and any(topic['issues'] for topic in release['topics'])
 
 
-def render_release_notes(project_name, project_version, include_next_release, page_id):
+def render_release_notes(project_key, project_version, include_next_release, page_id):
     from re import match
     from jinja2 import Template
     from pkg_resources import resource_string
@@ -93,55 +114,106 @@ def render_release_notes(project_name, project_version, include_next_release, pa
     from .jira_adapter import get_next_release_name_in_project
 
     template = Template(resource_string('infi.jira_cli', 'release_notes.html'))
-    project = get_project(project_name)
+    project = get_project(project_key)
     real_versions = [version for version in reversed(project.versions) if
                      match(r'[0-9\.]+', version.name) and not version.archived and
-                     version.released or (version.name == get_next_release_name_in_project(project_name) if include_next_release else False)]
+                     version.released or (version.name == get_next_release_name_in_project(project_key) if include_next_release else False)]
     releases = [get_release_notes_contents_for_specfic_version(project, version) for version in real_versions]
     exposed_releases = [release for release in releases if should_appear_in_release_notes(release)]
     return template.render(project=project, releases=exposed_releases, page_id=page_id)
 
 
-def publish_release_notes(project_name, project_version, include_next_release):
+def publish_release_notes(project_key, project_version, include_next_release):
     from .confluence_adapter import update_page_contents, get_release_notes_page_id
-    page_id = get_release_notes_page_id(project_name)
-    release_notes = render_release_notes(project_name, project_version, include_next_release, page_id)
+    page_id = get_release_notes_page_id(project_key)
+    release_notes = render_release_notes(project_key, project_version, include_next_release, page_id)
     update_page_contents(page_id, release_notes)
 
 
-def show_release_notes(project_name, project_version, include_next_release):
+def show_release_notes(project_key, project_version, include_next_release):
     from .confluence_adapter import get_release_notes_page_id
-    page_id = get_release_notes_page_id(project_name)
-    print render_release_notes(project_name, project_version, include_next_release, page_id)
+    page_id = get_release_notes_page_id(project_key)
+    print render_release_notes(project_key, project_version, include_next_release, page_id)
 
 
-def fetch_release_notes(project_name):
+def fetch_release_notes(project_key):
     from .confluence_adapter import get_page_contents, get_release_notes_page_id
-    print get_page_contents(get_release_notes_page_id(project_name))
+    print get_page_contents(get_release_notes_page_id(project_key))
 
 
-def notify_related_tickets(project_name, project_version):
-    from .jira_adapter import search_issues, issue_mappings, comment_on_issue
-    related_tickets = {}
-    for issue in search_issues("project={} AND fixVersion={!r} AND resolution=Fixed".format(project_name, project_version)):
-        for link in issue_mappings['IssueLinks'](issue):
-            if link.type.name in (u'Originates', ) and not link.inwardIssue.key.startswith(project_name.upper()):
-                related_tickets.setdefault(link.inwardIssue.key, list()).append(issue.key)
-    for key, issues_in_version in related_tickets.items():
-        comment_on_issue(key, "{} version {} which solves {} is out, go check it out".format(project_name, project_version, ' '.join(issues_in_version)))
+def notify_related_tickets(project_key, project_version, other_versions, dry_run):
+    def _build_jira_query_string():
+        from pkg_resources import parse_version
+        if other_versions:
+            versions.extend(sorted(set(other_versions + [project_version]), key=lambda version: parse_version(version)))
+            fix_version_string = 'fixVersion in ({})'.format(', '.join([repr(version) for version in versions]))
+        else:
+            fix_version_string = 'fixVersion={!r}'.format(project_version)
+        return "project={} AND {} AND resolution=Fixed".format(project_key, fix_version_string)
+
+    def _iter_related_tickets(issue):
+        for link in issue_mappings.IssueLinks(issue):
+            if not hasattr(link, 'inwardIssue'):
+                continue
+            if not link.type.name in (u'Originates', u'Clones', u'Relates'):
+                continue
+            if link.inwardIssue.key.startswith(project_key.upper()):
+                continue
+            yield link.inwardIssue
+
+    def find_issues_in_other_projects_that_are_pending_on_this_release():
+        related_tickets = {}
+        for issue in search_issues(_build_jira_query_string()):
+            for related_ticket in _iter_related_tickets(issue):
+                related_tickets.setdefault(get_issue(related_ticket.key), list()).append(issue)
+        return related_tickets
+
+    def _iter_related_remaining_open_issues(related_ticket):
+        for link in issue_mappings.IssueLinks(related_ticket):
+            if not hasattr(link, 'outwardIssue'):
+                continue
+            if not issue_mappings.Status(link.outwardIssue) in (u'Open', 'Reopened'):
+                continue
+            yield link.outwardIssue
+
+    def _build_comment(resolved_issues, unresolved_issues):
+        from jinja2 import Template
+        sort_issues = lambda issues: sorted(issues, key=lambda issue: issue.key)
+        notification_template = Template(NOTIFICATION_MESSAGE.strip())
+        return notification_template.render(project=project,
+                                            version=project_version,
+                                            other_versions=versions[:-1],
+                                            resolved_issues=sort_issues(resolved_issues),
+                                            unresolved_issues=sort_issues(unresolved_issues),
+                                            issue_mappings=issue_mappings)
+
+    from .jira_adapter import search_issues, issue_mappings, comment_on_issue, get_project, get_issue
+    project = get_project(project_key)
+    versions = []
+
+    related_tickets = find_issues_in_other_projects_that_are_pending_on_this_release()
+    for related_ticket, resolved_issues in sorted(related_tickets.items(), key=lambda item: item[0].key):
+        unresolved_issues = list(_iter_related_remaining_open_issues(related_ticket))
+        comment = _build_comment(resolved_issues, unresolved_issues)
+        if dry_run:
+            print "<--- COMMENT ON {0} STARTS HERE --->\n{1}\n<--- COMMENT ON {0} ENDS HERE ----->".format(related_ticket.key, comment)
+        else:
+            print 'commenting on %s' % related_ticket.key
+            comment_on_issue(related_ticket.key, comment)
 
 
 def do_work(arguments):
-    project_name = arguments['--project']
+    project_key = arguments['--project']
     project_version = arguments.get('--release')
     if arguments['show']:
-        show_release_notes(project_name, project_version, arguments['--include-next-release'])
+        show_release_notes(project_key, project_version, arguments['--include-next-release'])
     elif arguments['publish']:
-        publish_release_notes(project_name, project_version, arguments['--include-next-release'])
+        publish_release_notes(project_key, project_version, arguments['--include-next-release'])
     elif arguments['fetch']:
-        fetch_release_notes(project_name)
+        fetch_release_notes(project_key)
     elif arguments['notify']:
-        notify_related_tickets(project_name, project_version)
+        other_versions = arguments.get('<other-versions-included-in-this-releases>', list())
+        notify_related_tickets(project_key, project_version, other_versions, arguments['--dry-run'])
 
 
 def _jiject(argv, environ):
